function IPS(locationManager) {

	this.algorithmID = 1;
	this.debugIPS = false;
	this.locationManager = locationManager;

	this.setAlgorithmID = function (algorithmID) {
		this.algorithmID = algorithmID;
	}

	this.determineUserLocation = function (beaconData) {

		for (var i = 0; i < beaconData.length; i++) {
			beaconData[i]['r'] = Math.floor(beaconData[i]['rm'] / mapMetersPerCoordinatePoint);
		}
		showSignals(beaconData);

		if (beaconData.length == 1) {
			p ={};
			p['x'] = beaconData[0]['x'];
			p['y'] = beaconData[0]['y'];

		} else {

			// use algorithm
			p = this.calculatePositionByTrilateration(beaconData);
			// p = calculatePositionByTrilaterationGrid(beaconData);
		}

		if (p) {
			var coordPoint = [p['x'], p['y']];
			//alert(latlngPoint);
			// Create the event
			this.locationManager.handleLocationFound(coordPoint,beaconData);
		}
	}

	this.calculatePositionByTrilaterationGrid = function (beaconData) {
		beaconData.sort(function (a, b) {
			return a['rm'] - b['rm']
		});
		var im = 10;
		if (beaconData.length < im)
			im = beaconData.length;
		var xt = 0;
		var yt = 0;
		var b = 0;
		var m = {};
		for (var i = 0; i < im; i++) {
			r = Math.pow(beaconData[i]['r'], 2);
			//r= beaconData[i]['r'];
			x = beaconData[i]['x'];
			y = beaconData[i]['y'];
			xt = xt + x / r;
			yt = yt + y / r;
			b = b + 1 / r;
		}
		m['x'] = xt / b;
		m['y'] = yt / b;
		return m;
	}

	this.calculatePositionByTrilateration = function (beaconData) {

		var bestPairPoints = [];
		for (var i = 0; i < beaconData.length; i++) {
			var firstBeacon = beaconData[i];
			for (var k = i + 1; k < beaconData.length; k++) {
				var secondBeacon = beaconData[k];
				var bestPairPoint = this.getBestPointForBeaconPair(firstBeacon, secondBeacon);
				bestPairPoints.push(bestPairPoint);
			}
		}

		xm = 0;
		ym = 0;
		b = 0;
		var point;

		for (var i = 0; i < bestPairPoints.length; i++) {
			point = bestPairPoints[i];
			r = Math.pow(bestPairPoints[i]['r'], 2);
			xm = xm + point['x'] / r;
			ym = ym + point['y'] / r;
			b = b + 1 / r;
		}
		xm = xm / b;
		ym = ym / b;

		if (this.debugIPS) {
			console.log("position_x= " + xm);
			console.log("position_y= " + ym);
		}

		return {
			'x' : xm,
			'y' : ym
		};

	}

	this.getBestPointForBeaconPair = function (b1, b2) {
		m = {};
		x1 = b1['x'];
		y1 = b1['y'];
		r1 = b1['r'];

		x2 = b2['x'];
		y2 = b2['y'];
		r2 = b2['r'];

		if (x2 == x1)
			x2 = x2 + 0.1;
		if (y2 == y1)
			y2 = y2 + 0.1;

		k = (y2 - y1) / (x2 - x1);
		n = (x2 * y1 - x1 * y2) / (x2 - x1);
		if (this.debugIPS) {
			console.log("x1= " + x1);
			console.log("y1= " + y1);
			console.log("r1= " + r1);
			console.log("x2= " + x2);
			console.log("y2= " + y2);
			console.log("r2= " + r2);
			console.log("k= " + k);
			console.log("n= " + n);
		}

		a1 = Math.pow(k, 2) + 1;
		b1 = 2 * (k * (n - y1) - x1)
			c1 = Math.pow(n - y1, 2) + Math.pow(x1, 2) - Math.pow(r1, 2);
		if (this.debugIPS) {
			console.log("a1= " + a1);
			console.log("b1= " + b1);
			console.log("c1= " + c1);
		}

		dm1 = Math.pow(b1, 2) - 4 * a1 * c1;
		xm1_1 = (-b1 + Math.sqrt(dm1)) / (2 * a1);
		ym1_1 = k * xm1_1 + n;
		xm1_2 = (-b1 - Math.sqrt(dm1)) / (2 * a1);
		ym1_2 = k * xm1_2 + n;

		if (this.debugIPS) {
			console.log("xm1_1= " + xm1_1);
			console.log("ym1_1= " + ym1_1);
			console.log("xm1_2= " + xm1_2);
			console.log("ym1_2= " + ym1_2);
		}

		a2 = Math.pow(k, 2) + 1;
		b2 = 2 * (k * (n - y2) - x2)
			c2 = Math.pow(n - y2, 2) + Math.pow(x2, 2) - Math.pow(r2, 2);
		if (this.debugIPS) {
			console.log("a2= " + a2);
			console.log("b2= " + b2);
			console.log("c2= " + c2);
		}

		dm2 = Math.pow(b2, 2) - 4 * a2 * c2;
		xm2_1 = (-b2 + Math.sqrt(dm2)) / (2 * a2);
		ym2_1 = k * xm2_1 + n;
		xm2_2 = (-b2 - Math.sqrt(dm2)) / (2 * a2);
		ym2_2 = k * xm2_2 + n;
		if (this.debugIPS) {
			console.log("xm2_1= " + xm2_1);
			console.log("ym2_1= " + ym2_1);
			console.log("xm2_2= " + xm2_2);
			console.log("ym2_2= " + ym2_2);
		}

		// find 2 closest points
		d1 = Math.pow((xm2_1 - xm1_1), 2) + Math.pow((ym2_1 - ym1_1), 2);
		d2 = Math.pow((xm2_2 - xm1_1), 2) + Math.pow((ym2_2 - ym1_1), 2);
		d3 = Math.pow((xm2_1 - xm1_2), 2) + Math.pow((ym2_1 - ym1_2), 2);
		d4 = Math.pow((xm2_2 - xm1_2), 2) + Math.pow((ym2_2 - ym1_2), 2);
		if (this.debugIPS) {
			console.log("d1= " + d1);
			console.log("d2= " + d2);
			console.log("d3= " + d3);
			console.log("d4= " + d4);
		}

		d = Math.min(d1, d2, d3, d4);
		if (this.debugIPS) {
			console.log("d= " + d);
		}
		distance = Math.sqrt(Math.pow(x2 - x1, 2) + Math.pow(y2 - y1, 2));
		if (d == d1) {
			if (distance < Math.abs(r1 - r2)) {
				if (r1 < r2) {
					m['x'] = xm1_1;
					m['y'] = ym1_1
				} else {
					m['x'] = xm2_1;
					m['y'] = ym2_1
				}
			} else if (distance > Math.abs(r1 + r2)) {
				m1 = Math.pow(r1, 2);
				m2 = Math.pow(r2, 2);
				m['x'] = (xm1_1 / m1 + xm2_1 / m2) / (1 / m1 + 1 / m2);
				m['y'] = (ym1_1 / m1 + ym2_1 / m2) / (1 / m1 + 1 / m2);
			} else {
				m['x'] = xm1_1 + (xm2_1 - xm1_1) / 2;
				m['y'] = ym1_1 + (ym2_1 - ym1_1) / 2;

			}

		} else if (d == d2) {
			if (distance < Math.abs(r1 - r2)) {
				if (r1 < r2) {
					m['x'] = xm1_1;
					m['y'] = ym1_1
				} else {
					m['x'] = xm2_2;
					m['y'] = ym2_2
				}
			} else if (distance > Math.abs(r1 + r2)) {
				m1 = Math.pow(r1, 2);
				m2 = Math.pow(r2, 2);
				m['x'] = (xm1_1 / m1 + xm2_2 / m2) / (1 / m1 + 1 / m2);
				m['y'] = (ym1_1 / m1 + ym2_2 / m2) / (1 / m1 + 1 / m2);
			} else {
				m['x'] = xm1_1 + (xm2_2 - xm1_1) / 2;
				m['y'] = ym1_1 + (ym2_2 - ym1_1) / 2;
			}

		} else if (d == d3) {
			if (distance < Math.abs(r1 - r2)) {
				if (r1 < r2) {
					m['x'] = xm1_2;
					m['y'] = ym1_2
				} else {
					m['x'] = xm2_1;
					m['y'] = ym2_1
				}
			} else if (distance > Math.abs(r1 + r2)) {
				m1 = Math.pow(r1, 2);
				m2 = Math.pow(r2, 2);
				m['x'] = (xm1_2 / m1 + xm2_1 / m2) / (1 / m1 + 1 / m2);
				m['y'] = (ym1_2 / m1 + ym2_1 / m2) / (1 / m1 + 1 / m2);
			} else {
				m['x'] = xm1_2 + (xm2_1 - xm1_2) / 2;
				m['y'] = ym1_2 + (ym2_1 - ym1_2) / 2;
			}
		} else if (d == d4) {
			if (distance < Math.abs(r1 - r2)) {
				if (r1 < r2) {
					m['x'] = xm1_2;
					m['y'] = ym1_2
				} else {
					m['x'] = xm2_2;
					m['y'] = ym2_2
				}
			} else if (distance > Math.abs(r1 + r2)) {
				m1 = Math.pow(r1, 2);
				m2 = Math.pow(r2, 2);
				m['x'] = (xm1_2 / m1 + xm2_2 / m2) / (1 / m1 + 1 / m2);
				m['y'] = (ym1_2 / m1 + ym2_2 / m2) / (1 / m1 + 1 / m2);
			} else {
				m['x'] = xm1_2 + (xm2_2 - xm1_2) / 2;
				m['y'] = ym1_2 + (ym2_2 - ym1_2) / 2;

			}
		}
		if (this.debugIPS) {
			console.log("mx= " + m['x']);
			console.log("my= " + m['y']);

		}

		if (r1 < r2)
			m['r'] = r1;
		else
			m['r'] = r2
				return m;

	}

}