

var mapViewerDefaultOptions = {
	osm : false,
	loadCustomTiles : false,
	tilesUrl : "./assets/tiles/{z}/{x}/{y}.png",
	mapExtent : [0,0,0,0],
	zoom:14,
	navZoom:16,
	mapMinZoom : 14,
	mapMaxZoom : 19,
	enableDebugConsole:false,
	showModeSwitchButton : true,
	mapMode :3,
	mapNavMode :2,
	showPanButton : false,
	showRotateToggleButton : false,
	showResumeButton: true,
	showCalibrationInfo : false,
	showSearchButton : true,
	rotateMode : 'rotate-map', //rotate-map or rotate-icon
	showCompass : true,
	showEye : false,
	showAR : false,
	showFlipOrderButton : false,
	mapMetersPerCoordinatePoint : 1 / 5, // map scale , automatically calculated is map is geo-positioned
	aproachingNode : 3, // meters before path turn to show turn directions
	onPath : 10, // meters away from path when trigger return to path directions
	avarageWalkingSpeed : 1, // average walking speed meters per second
	orientationTrigger : 30, // min orientation change in degrees triggering  directions update
	turnInstructionCrossingMinAngle : 0,
	turnInstructionBendMinAngle : 89,
	lookRadius : 100, // scan radius in meters
	positioningMode : 'external', // positioning mode to use, valid options are 'test', 'ips', 'external'
	language : 'en',
	arrivedDistance : 15,
	approachingDistance : 50,
	reroutSkipCountMax : 7,
	iconScaleFactor : 0.2,
	magnaticPoleOffset : 8,
	magnaticPoleOffsetAndroid : 8,
	kioskMode : false,
};


var mapMetersPerCoordinatePoint = 1 / 5;
var viewableBounds;
var mapCenter;
var map;
var navigatorCore;
var appConnector;
var popup;
var debugConsoleEnabled;
var language;

function NoClickDelay(el) {
	this.element = el;
	if( window.Touch ) this.element.addEventListener('touchstart', this, false);
}

NoClickDelay.prototype = {
	handleEvent: function(e) {
		switch(e.type) {
			case 'touchstart': this.onTouchStart(e); break;
			case 'touchmove': this.onTouchMove(e); break;
			case 'touchend': this.onTouchEnd(e); break;
		}
	},

	onTouchStart: function(e) {
		e.preventDefault();
		this.moved = false;
		this.element.addEventListener('touchmove', this, false);
		this.element.addEventListener('touchend', this, false);
	},

	onTouchMove: function(e) {
		this.moved = true;
	},

	onTouchEnd: function(e) {
		this.element.removeEventListener('touchmove', this, false);
		this.element.removeEventListener('touchend', this, false);

		if( !this.moved ) {
			// Place your code here or use the click simulation below
			var theTarget = document.elementFromPoint(e.changedTouches[0].clientX, e.changedTouches[0].clientY);
			if(theTarget.nodeType == 3) theTarget = theTarget.parentNode;

			var theEvent = document.createEvent('MouseEvents');
			theEvent.initEvent('click', true, true);
			theTarget.dispatchEvent(theEvent);
		}
	}
};
var  deviceOrientationEventName = 'deviceorientation'
var is_android = false;

function isAndroid() {
	return is_android;
}

function updateOption(key,value) {
	mapViewerOptions[key]=value;
}



function initMap(viewerOptions) {
	var ua = navigator.userAgent;
	is_android = (ua.indexOf('Android') > -1);

	if (!isAndroid()) 
		new NoClickDelay(document.getElementById('map'));
	else deviceOrientationEventName = 'deviceorientationabsolute'
	var navData = {
	//	'backboneJSON' : appConnector.getBackboneJSON(),
	//	'routingJSON' : appConnector.getRoutingJSON(),
	//	'labelsJSON' : appConnector.getLabelsJSON(),
	//	'staticMarkersJSON' : appConnector.getStaticMarkersJSON(),
	//	'pbMarkersJSON' : appConnector.getPinboardMarkersJSON(),
	//	'activePBMarkersJSON' : appConnector.getActivePinboardMarkersJSON(),
	//	'eventsJSON' : appConnector.getEventsJSON(),
	}
	mapViewerOptions = $.extend(true, {}, mapViewerDefaultOptions, viewerOptions);
	//console.log(JSON.stringify(mapViewerOptions));
	language = mapViewerOptions.language;
	initSearchOptionsDef();
	debugConsoleEnabled = mapViewerOptions.enableDebugConsole;
	initMapControls(mapViewerOptions);
	initWrapperControls();
	map = createGeoPositionedMap();
	//map = createRasterImageMap();

	// init layers
	vectorSourceLabels = new ol.source.Vector({});
	vectorLayerLabels = new ol.layer.Vector({
			source : vectorSourceLabels,
			style : labelStyle()
		});
	map.addLayer(vectorLayerLabels);

	var vectorSourceIPSSignals = new ol.source.Vector({});
	var vectorLayerIPSSignals = new ol.layer.Vector({
			source : vectorSourceIPSSignals
		});
	map.addLayer(vectorLayerIPSSignals);

	var vectorSourceNavigationGrid = new ol.source.Vector({});
	var vectorLayerNavigationGrid = new ol.layer.Vector({
			source : vectorSourceNavigationGrid
		});
	map.addLayer(vectorLayerNavigationGrid);

	var vectorSourceNavigation = new ol.source.Vector({});
	var vectorLayerNavigation = new ol.layer.Vector({
			source : vectorSourceNavigation
		});
	vectorLayerNavigation.set('id', 'nav_layer')
	map.addLayer(vectorLayerNavigation);
  
	var vectorSourceLocation = new ol.source.Vector({});
	var vectorLayerLocation = new ol.layer.Vector({
			source : vectorSourceLocation,
			updateWhileInteracting: true,
		});
	

	var vectorSourceMarkers = new ol.source.Vector({});
	var vectorLayerMarkers = new ol.layer.Vector({
			source: vectorSourceMarkers,
			style: markerStyle()
		});
	if (mapViewerOptions.kioskMode) {
		map.addLayer(vectorLayerMarkers);
		map.addLayer(vectorLayerLocation);
	} else {
		map.addLayer(vectorLayerLocation);
		map.addLayer(vectorLayerMarkers);
	}

	initPopupDisplay();
	initNavPopupDisplay();
	initDebugConsole(debugConsoleEnabled);
	var initialLocation = mapCenter;

	navigatorCore = new Navigator(mapViewerOptions, vectorLayerLabels, vectorLayerNavigationGrid, vectorLayerNavigation, vectorLayerLocation, vectorLayerMarkers, mapMetersPerCoordinatePoint, navData);
	navigatorCore.initNavigator();
	navigatorCore.orientationManager.setRotationCentre(mapCenter);
	navigatorCore.userLocationDisplay.setDisplayUserLocation(true);

	//navigatorCore.locationManager.handleLocationFound(initialLocation);
	//if (mapViewerOptions.showCalibrationInfo == true) {
	//	$('#calibrate-modal-sm').modal('toggle');
	//}
	removeTourUI()
		if (mapViewerOptions.rotateMode=="rotate-map"){
			navigatorCore.orientationManager.setAutoRotateMap(true)
		} else {
			navigatorCore.orientationManager.setAutoRotateMap(false)
		}
		initFilterOptions();
	setMode(mapViewerOptions.mapMode);
	initModeMonitor();	
	initShowOnMapTempMarker(vectorSourceMarkers);
	map.getView().setCenter(mapCenter);
	if (mapViewerOptions.kioskMode){
		console.log("kiosk mode");

		 $('#trailItemsSearch').on('shown.bs.modal', function () {
		// $(".modal-backdrop.in").hide();
		   $('.modal-backdrop').remove();

		})
		var itemData = navigatorCore.navItemFLayer.boundItemsData[mapViewerOptions.kioskMarker];
		console.log(itemData);
		if(itemData){
			map.getView().setZoom(mapViewerOptions.kioskZoom);
			map.getView().setCenter(itemData.lonLat);
			appConnector.updateUserLocationLonLat(itemData.lonLat);
			navigatorCore.orientationManager.setIconRotation(mapViewerOptions.kioskViewCompassBearingDeg);
			//navigatorCore.deviceOrientationChanged(mapViewerOptions.kioskViewCompassBearingDeg);
		}
	}
}

function createGeoPositionedMap() {
	//	var imageSize = [0, 0, 1644, 2070];
//	var mapExtent = [ 151.211224098285,-33.8723833178949,151.223712432547,-33.8555827396741];
	var mapExtent = mapViewerOptions.mapExtent;
	mapCenter = ol.proj.fromLonLat([mapExtent[0]+(mapExtent[2]-mapExtent[0])/2, mapExtent[1]+(mapExtent[3]-mapExtent[1])/2]);

	if (mapViewerOptions.mapCenter ) mapCenter = ol.proj.fromLonLat(mapViewerOptions.mapCenter );
		
	var mapExtentPr = ol.proj.transformExtent(mapExtent, 'EPSG:4326', 'EPSG:3857');
	//console.log("mapExtentPr " + mapExtentPr)

	viewableBounds = mapExtentPr;
	var mapMinZoom =mapViewerOptions.mapMinZoom;
	var mapMaxZoom = mapViewerOptions.mapMaxZoom;
	var zoom = mapViewerOptions.zoom;
	var osmLayer;
	if (mapViewerOptions.osm){
		 osmLayer = new ol.layer.Tile({
				source : new ol.source.OSM()
			});
	}
	var image = new ol.layer.Image({
			source : new ol.source.ImageStatic({
				url : './assets/map.png',
				projection : 'EPSG:4326',
				//imageSize : imageSize,
				imageExtent : mapExtent
			})
		})
		var layer = new ol.layer.Tile({
			preload: Infinity,
			extent : mapExtentPr,
			source : new ol.source.XYZ({
				url : mapViewerOptions.tilesUrl, 
				minZoom : mapMinZoom,
				maxZoom : mapMaxZoom
			})
		});
	var scaleLineControl = new ol.control.ScaleLine({
			units : 'metric'
		});
	var map = new ol.Map({
			controls : ol.control.defaults({
				zoom : mapViewerOptions.kioskMode,
				attribution : false,
				rotate : false
			}).extend([
					scaleLineControl
				]),
			interactions: ol.interaction.defaults({altShiftDragRotate:false, pinchRotate:false, dragPan:true}),
			target : 'map',
			layers : [
				//osm,
				//image
			//	layer
			],
			view : new ol.View({
				center : mapCenter,
				zoom : zoom,
				minZoom : mapMinZoom,
				maxZoom : mapMaxZoom,
				extent : mapExtentPr,

			})
		});
	if (mapViewerOptions.osm){
	map.addLayer(osmLayer)
	}
	map.addLayer(layer)
	mapMetersPerCoordinatePoint = calculateMetersPerCoordinatePoint([151.238783, -33.846106], [151.244934, -33.846106])
		//console.log('coord= '+ol.proj.fromLonLat([151.238783, -33.846106]));
		//console.log('coord= '+ol.proj.fromLonLat([151.244934, -33.846106]));
		return map;

}




function calculateMetersPerCoordinatePoint(c1, c2) {
	var wgs84Sphere = new ol.Sphere(6378137);
	// var c1_pr = ol.proj.transform(c1, projection, 'EPSG:4326');
	// var c2_pr = ol.proj.transform(c2, projection, 'EPSG:4326');
	var length = wgs84Sphere.haversineDistance(c1, c2);
	// console.log('length= '+length);
	var p1 = ol.proj.fromLonLat(c1);
	var p2 = ol.proj.fromLonLat(c2);
	var scale = length / Math.abs(p1[0] - p2[0])
		//  console.log('scale= '+scale);
		// var scale =
		return scale;
}

function createRasterImageMap() {
	var extent = [0, 0, 1232, 901];
	var mapMinZoom = 1;
	var mapMaxZoom = 5;

	//mapCenter = ol.extent.getCenter(extent),
	mapCenter = [594.34375, 371.765625]

	viewableBounds = extent;

	var projection = new ol.proj.Projection({
			code : 'xkcd-image',
			units : 'pixels',
			extent : extent
		});

	var map = new ol.Map({
			controls : ol.control.defaults({
				zoom : false,
				attribution : false,
				rotate : false,
				altShiftDragRotate:false,
				pinchRotate:false
			}).extend([
					//new app.ShowMyLocationControl(),
					//	new app.ShowSignalsControl(),
					//new app.RotationModeControl(),
					//	new app.NavigationGridModeControl()

				]),
			interactions: ol.interaction.defaults({altShiftDragRotate:false, pinchRotate:false, dragPan:true}),
			layers : [
				new ol.layer.Image({
					source : new ol.source.ImageStatic({
						url : './assets/map.png',
						projection : projection,
						imageExtent : extent
					})
				})
			],
			target : document.getElementById('map'),
			view : new ol.View({
				projection : projection,
				center : mapCenter,
				extent : extent,
				zoom : mapMinZoom,
				maxZoom : mapMaxZoom,
				minZoom : mapMinZoom,
			})
		});
	return map;

}

function initMapControls(mapViewerOptions) {
	var bodyEl = $(document.body);

	if (mapViewerOptions.showPanButton) {
		var a = jQuery('<a/>', {
				//id : 'foo',
				href : '#',
				id : "btnPanToMyLocation",
				class : "pan-to-my-l"
			}).appendTo(bodyEl);

		$("#btnPanToMyLocation").bind("click", function (event, ui) {
			if (navigatorCore.locationManager.getLastKnownLocation())
				map.getView().setCenter(navigatorCore.locationManager.getLastKnownLocation());
			else {
				map.getView().setCenter(mapCenter);
			}

		});
	};

	if (mapViewerOptions.showRotateToggleButton == true) {
		
		if (mapViewerOptions.rotateMode=="rotate-map"){
				var a = jQuery('<a/>', {
				//id : 'foo',
				href : '#',
				id : "btnRotateModeToggle",
				class : "rotate-map"
			}).appendTo(bodyEl);
			
		} else {
				var a = jQuery('<a/>', {
				//id : 'foo',
				href : '#',
				id : "btnRotateModeToggle",
				class : "rotate-icon"
			}).appendTo(bodyEl);
	
		}
		
		var title = $('#calibrate-modal-sm').find('#calibrate-modal-title')
		title.text(langString (language, 'calibrate_modal_name'));
		
	    var body = $('#calibrate-modal-sm').find('#calibrate-modal-body');
		body.html(langString (language, 'calibrate_modal_body'));
		
		$("#btnRotateModeToggle").bind("click", function (event, ui) {

			//$("#btnRotateModeToggle").attr('class') == 'rotate-map'

			if (!($("#btnRotateModeToggle").attr('class') == 'rotate-map')) {
				navigatorCore.orientationManager.setAutoRotateMap(true)
				$("#btnRotateModeToggle").attr('class', 'rotate-map');
				if (isAndroid() && mapViewerOptions.showCalibrationInfo == true) {
					$('#calibrate-modal-sm').modal('toggle');
				}
	
			} else {
				navigatorCore.orientationManager.setAutoRotateMap(false)
				$("#btnRotateModeToggle").attr('class', 'rotate-icon');
			}

		});

	};

		if (mapViewerOptions.showSearchButton == true) {
		var a = jQuery('<a/>', {
				//id : 'foo',
				href : '#',
				id : "btnSearch",
				class : "search-icon"
			}).appendTo(bodyEl);

		a.bind("click", function (event, ui) {
		//alert('working')
		$('#search_options').show()
		$('#keyword_search').hide()

		
		
		
		var modal = $('#trailItemsSearch')
		var div = modal.find('#items-list-display');
		$(div).empty();
		
		if ( mapViewerOptions.kioskMode){
			$('#trailItemsSearch').modal({backdrop: 'static', keyboard: false})  
		} else {
			$('#trailItemsSearch').modal('toggle');
		}
		
		});
		var title = $('#trailItemsSearch').find('#search-modal-title')
		title.text(langString (language, 'search_modal_name'));

	};
	

		if (mapViewerOptions.showFlipOrderButton == true) {
		//var titleBar = 	$('#tour_overlay-titlebar')
		var a = jQuery('<a/>', {
				//id : 'foo',
				href : '#',
				id : "flipOrder",
				class : "flipOrder"
			}).appendTo(bodyEl);

		a.bind("click", function (event, ui) {
		flipOrder()
		});
		
	};
	
	
	if (mapViewerOptions.showCompass) {

	};

	if (mapViewerOptions.showEye) {
		var a = jQuery('<a/>', {
				//id : 'foo',
				href : '#',
				id : "btnEye1",
				class : "button-eye"
			}).appendTo(bodyEl);
	var title = $("#eye_overlay").find('#eye_overlay_title')
	title.text(langString (language, 'eye_scanner_name'));
			
	$("#btnEye1").bind("click", function (event, ui) {
		dismissEye();
		$("#navigation_overlay").hide();
		//alert('show eye');
		$("#eye_overlay").show();

		navigatorCore.lookAssistant.scanForItems();
	});
	
	};

	if (mapViewerOptions.showAR) {
		var a = jQuery('<a/>', {
				//id : 'foo',
				href: "#",
				id: "btnAR",
				class: "button-ar",
			}).appendTo(bodyEl);
		$("#btnAR").bind("click", function (event, ui) {
			appConnector.buttonARClicked();
		});

		$("#btnAR").hide()
	};
	
	if (mapViewerOptions.showModeSwitchButton == true) {
		
			var a = jQuery('<a/>', {
				//id : 'foo',
				href : '#',
				id : "btnModeSwitchToggle",
			}).appendTo(bodyEl);
			
				var span= jQuery('<span/>', {
				//id : 'foo',
				//href : '#',
				id : "compass_display",
				class : "compass_display"
			}).appendTo(a);
			
		$("#btnModeSwitchToggle").bind("click", function (event, ui) {
			if (mapMode == mode_lockAndRotateMap)
			{
			 setMode(mode_unlockAndRotateIcon);

			} else if (mapMode == mode_unlockAndRotateIcon)
			{
			 setMode(mode_lockAndRotateIcon);	

			} else if (mapMode == mode_lockAndRotateIcon)
			{
			setMode(mode_lockAndRotateMap);

			}

		});

	};
	
	if (mapViewerOptions.showResumeButton == true) {
		
			var a = jQuery('<a/>', {
				text : langString (language, 'resume'),
				href : '#',
				id : "btnModeResume",
				class : "btnModeResume"
			}).appendTo(bodyEl);

	$("#btnModeResume").hide();	
		$("#btnModeResume").bind("click", function (event, ui) {
			resumeNavigationMode()

		});

	};

		/*	
			var a = jQuery('<a/>', {
				text : 'compass',
				href : '#',
				id : "btnRotateCompass",
				class : "btnRotateCompass"
			}).appendTo(bodyEl);

var count=0;
		$("#btnRotateCompass").bind("click", function (event, ui) {
			count++;
			//navigatorCore.orientationManager.deviceOrientationListener.set('alpha', Math.PI/45 *count );
            navigatorCore.orientationManager.rotateMapSim(( Math.PI/45) *count );
			
			//navigatorCore.orientationManager.rotateCompass(navigatorCore.orientationManager.lastKnownRotationDeg +5)

		});
	
			var a = jQuery('<a/>', {
				text : 'compassA',
				href : '#',
				id : "btnRotateCompassA",
				class : "btnRotateCompassA"
			}).appendTo(bodyEl);

var count=0;
		$("#btnRotateCompassA").bind("click", function (event, ui) {
			count++;
			//navigatorCore.orientationManager.deviceOrientationListener.set('alpha', Math.PI/45 *count );
            navigatorCore.orientationManager.rotateIconSim(( Math.PI/45) *count );
			
			//navigatorCore.orientationManager.rotateCompass(navigatorCore.orientationManager.lastKnownRotationDeg +5)

		});	
*/		

}

var mapMode;
var mode_lockAndRotateMap=1;
var mode_lockAndRotateIcon=2;
var mode_unlockAndRotateIcon=3;

var lockMapOnUserLocation = false;

function setMode(newMode) {
	if (mapMode == newMode) {
		//alert('same');
		return;
	} else {
		//	alert('re-set');
	}
	mapMode = newMode;
	//console.log(map.getView().getZoom() )
	if (mapMode == mode_lockAndRotateMap) {
		$("#btnModeSwitchToggle").attr('class', 'lockAndRotateMap');
		if (navigatorCore.canLock())
		map.getView().setCenter(navigatorCore.locationManager.getLastKnownLocation());
		//else 
		//map.getView().setCenter(mapCenter);
		navigatorCore.orientationManager.setAutoRotateMap(true)
		lockMapOnUserLocation = true;
	//	if (map.getView().getZoom() < mapViewerOptions.navZoom-1)
		//	map.getView().setZoom(mapViewerOptions.navZoom);

	} else if (mapMode == mode_lockAndRotateIcon) {
		$("#btnModeSwitchToggle").attr('class', 'lockAndRotateIcon');
		if (navigatorCore.canLock())
		map.getView().setCenter(navigatorCore.locationManager.getLastKnownLocation());
		////else 
		//map.getView().setCenter(mapCenter);
		navigatorCore.orientationManager.setAutoRotateMap(false)
		lockMapOnUserLocation = true;
	//	if (map.getView().getZoom() < mapViewerOptions.navZoom-1)
	//		map.getView().setZoom(mapViewerOptions.navZoom);


	} else if (mapMode == mode_unlockAndRotateIcon) {
		$("#btnModeSwitchToggle").attr('class', 'unlockAndRotateIcon');
		navigatorCore.orientationManager.setAutoRotateMap(false)
		lockMapOnUserLocation = false;
	}
	
	if ( lockMapOnUserLocation == false && navigatorCore.canLock()){
	$("#btnModeResume").show();	
		
	} else {
	$("#btnModeResume").hide();		
	}

}

function resumeNavigationMode() {
	setMode(mapViewerOptions.mapNavMode)
}

function getItemTargetModal(itemID) {
	
	var target_id = itemID;
			//	console.log(target_id);
				var itemData = navigatorCore.navItemFLayer.boundItemsData[target_id];
					//	console.log(itemData);
				if(itemData) {
					var isTourItem= tourItems && tourItems[target_id]!=undefined;
					if (isTourItem) {
					//	console.log(target_id);
						//var modal = $('#trailItemsSearch')
						return '#tourItems'
					} else if ('temp'!=itemData['id'])  {
						return  '#item-details-modal-sm';
						
					} else return '#temp-item-details-modal-sm'
					
					
				}
			
	return  '#item-details-modal-sm';
}


function initPopupDisplay() {

	popup = new ol.Overlay.Popup();
	map.addOverlay(popup);

	map.on('click', function (evt) {
		 $(popup.container).removeClass("tour");
		 $(popup.container).removeClass("normal");
		 popup.panMapIfOutOfView=true;
		// Hide existing popup and reset it's offset
		popup.hide();

		// Attempt to find a feature in one of the visible vector layers
		var feature = map.forEachFeatureAtPixel(evt.pixel, function (feature, layer) {
				return feature;
			});

		if (feature && feature.get("type") == "marker-with-popup") {

			//console.log(feature.get("type"));
			var coord = feature.getGeometry().getCoordinates();
			var props = feature.getProperties();
			//console.log('You popup_content: ' + feature.get('popup_content'));

			//popup.show(coord, createMapPoupHTML(feature.getId()));
			var target_id = feature.getId();
			//alert ("target_id")
			var isTourItem= tourItems && tourItems[target_id]!=undefined;
			if (isTourItem) {	
			
			   // $(popup.container).addClass("tour");
				//popup.show(coord, createMapPoupHTMLRTour(feature.getId())) 
				var itemData = navigatorCore.navItemFLayer.boundItemsData[target_id];
				var slideIndex = sortedTourItems.indexOf(itemData) 
				sliderTour.goToSlide(slideIndex+1);
				}
			else {	
			   $(popup.container).addClass("normal");
			   popup.show(coord, createMapPoupHTML(feature.getId()))
			 
			}		
					

		}
		if (feature && feature.get("type") == "temp-marker-with-popup") {

			//console.log(feature.get("type"));
			var coord = feature.getGeometry().getCoordinates();
			var props = feature.getProperties();
			//console.log('You popup_content: ' + feature.get('popup_content'));
			//popup.container.addClass("normal");
			popup.show(coord, createTempMarkerMapPoupHTML(feature['itemData']));

		}

	});


	// change mouse cursor when over marker
	map.on('pointermove', function (e) {
		if (e.dragging) {
			//$(element).popover('destroy');
			return;
		}
		//	var pixel = map.getEventPixel(e.originalEvent);
		//	var hit = map.hasFeatureAtPixel(pixel);
		//	map.getTarget().style.cursor = hit ? 'pointer' : '';
	});

}

var popupNav;

function initNavPopupDisplay() {

	popupNav = new ol.Overlay.Popup();
	map.addOverlay(popupNav);
   $(popupNav.container).addClass("tour");

}
function initModeMonitor() {

	$( map ).on('click', function (evt) {	
	setMode(mode_unlockAndRotateIcon);	
	});
		$( map ).on('pointermove', function (evt) {
//alert("movestart")			
	setMode(mode_unlockAndRotateIcon);	
	});
}

function initWrapperControls() {
	$('#tour_overlay').hide();	
	$("#navigation_overlay").hide();
	$("#selection_overlay").hide();

	$("#btnSelect").bind("click", function (event, ui) {
		$("#navigation_overlay").show();
		$("#selection_overlay").show();
	});

	$("#btnSelect").bind("click", function (event, ui) {
		$("#navigation_overlay").show();
		$("#selection_overlay").show();
	});

	$("#btnNavigate").bind("click", function (event, ui) {
		$("#navigation_overlay").show();
	});

	$("#btnDismissNav").bind("click", function (event, ui) {
		navigatorCore.navPathAssistant.stopNavigationAssistance();
		$("#navigation_overlay").hide();
	});

	$("#btnDismissSelection").bind("click", function (event, ui) {
		$("#navigation_overlay").hide();
		$("#selection_overlay").hide();
		$("#main_bar").show();
	});

	$("#eye_overlay").hide();
   


	$("#btnDismissEye").bind("click", function (event, ui) {
		dismissEye();

	});

$('#item-details-modal-sm').on('show.bs.modal', function (event) {
	var a = $(event.relatedTarget) // Button that triggered the modal

	var targer_id = $(event.relatedTarget).attr('target_id');

	var itemData = navigatorCore.navItemFLayer.boundItemsData[targer_id];
	var itemID = itemData['id'];

	var isTourItem= tourItems && tourItems[itemID]!=undefined;
	
	var modal = $(this)

	var div = modal.find('#item-modal-cont')

	$(div).empty();
	
	var a;
	if (itemData['location_id']) {
		a = jQuery('<a/>', {
				//id : 'foo',
				href: "#",
				onclick : "appConnector.locationViewMoreClicked(event);",
				location_id : itemData['location_id'],
				id: "item-map-modal-popup-link",
				class: "item-map-popup-link"
			}).appendTo(div);

	} else  {
			a = jQuery('<div/>', {
			//id : 'foo',
			//href : '#',
			id : "item-map-modal-popup-link",
			class : "item-map-popup-link"
		}).appendTo(div);
		
	}


	//a.attr('target-id', itemID);


	var divImg = jQuery('<div/>', {
			href : '#',
			id : "item-map-popup-img",
			class : "item-map-popup-img"
		}).appendTo(a);

	var img = jQuery('<img/>', {
			id : "item-map-popup-img-img",
			class : "item-map-popup-img-img",
			src : itemData['popupPreviewURL']
		}).appendTo(divImg);
	if (itemData['location_id']) {
		$(img).attr('location_id', itemData['location_id']);
	}		
		
	var divTitleBar = jQuery('<div/>', {
			href : '#',
			id : "item-map-popup-title-bar",
			class : "item-map-popup-title-bar"
		}).appendTo(div);

	var span = jQuery('<span/>', {
			id : "item-map-popup-text",
			class : "item-map-popup-text",
			text : itemData['popupHeading']
		}).appendTo(divTitleBar);



	var nextEvent = navigatorCore.navItemFLayer.getNextEvent(itemID);
	if (nextEvent) {

		var span1 = jQuery('<span/>', {
				id : "item-map-popup-event-type",
				class : "item-map-popup-event-type",
				text : nextEvent.type + ' at: '
			}).appendTo(divTitleBar);

		var span3 = jQuery('<span/>', {
				id : "item-map-popup-event-time",
				class : "item-map-popup-event-time",
				text : nextEvent.start_time
			}).appendTo(divTitleBar);

	}
	jQuery('<br/>', {}).appendTo(span);

	var distanceTo = navigatorCore.findDistanceToItemM(itemData['id']);
	if (distanceTo)
		var spanDistance = jQuery('<span/>', {
				id : "item-map-popup-distance",
				class : "item-map-popup-distance",
				text : distanceTo
			}).appendTo(divImg);
	
	var span_bottom_toolbar = jQuery('<span/>', {
			id : "item-map-popup-bottom-toolbar",
			class : "item-map-popup-bottom-toolbar"
		}).appendTo(div);

	//var a_dir = jQuery('<a/>', {
	//		href : '#',
	//		id : "item-map-popup-get-directions-link",
	//		class : "item-map-popup-get-directions-link",
	//		onclick : "getDirections(event)",
	///		text : langString (language,'directions')
	//	}).appendTo(span_bottom_toolbar);

	//a_dir.attr('target-id', itemID);
	var link = jQuery('<a/>', 
			{
						href : "#",
						class : "item-map-popup-show-on-map",
						onclick : "showStaticMarkerOnMap('"+ itemData['id']+"')",
						text : langString (language, 'show_on_map')
			}).appendTo(span_bottom_toolbar);
	
	
	
	if (itemData['location_id']) {
		var link = jQuery('<a/>', {
				href: "#",
				onclick : "appConnector.locationViewMoreClicked(event);",
				location_id : itemData['location_id'],
				class: "item-map-popup-view-more",
				id: "item-map-popup-view-more-modal",
				//class : "item-map-popup-get-directions-link",
				text: 'Info'
			}).appendTo(span_bottom_toolbar);

	}

	if (canBeAddedToMyDay(itemData.marker_type)) {

		var link = jQuery('<a/>', {
				href : "#",
				onclick : "appConnector.locationAddToMyDayClicked(event);",
				location_id : itemData['location_id'],
				markerid : itemData['id'],
				class : "item-map-popup-add-to-my-day-link",
				id : "item-map-popup-modal-add-to-my-day-link",
				text : langString (language, 'add')
			}).appendTo(span_bottom_toolbar);

	}

})




	$('#trailItems').on('show.bs.modal', function (event) {
		//var a = $(event.relatedTarget) // Button that triggered the modal
		//console.log("you clicked " + );
		//	var targer_id = $(event.relatedTarget).attr('target_id');
		//	var itemData = navigatorCore.navItemFLayer.boundItemsData[targer_id];

		var modal = $(this)
			var div = modal.find('#items-list-display');
		$(div).empty();

		var boundItemsData = navigatorCore.navItemFLayer.boundItemsData;
		for (var key in boundItemsData) {
			var itemData = boundItemsData[key];

			var item_div = jQuery('<div/>', {
					//	href : '#',
					class : "list-group-item",
					//	onclick : "selectItem()",

				}).appendTo(div);

			item_div.attr('target-id', itemData.id);

			var div_img_cont = jQuery('<div/>', {
					//	href : '#',
					class : "item-list-img-cont",
					//	onclick : "selectItem()",

				}).appendTo(item_div);

			var img = jQuery('<img/>', {
					class : "pull-left item-list-img",
					src : itemData['popupPreviewURL']
				}).appendTo(div_img_cont);

			var distanceTo = navigatorCore.findDistanceToItemM(itemData['id']);

			var span = jQuery('<span/>', {
					class : "label label-success label-pill",
					text : distanceTo
				}).appendTo(div_img_cont)

				var div_pr_cont = jQuery('<div/>', {
					//	href : '#',
					class : "item-list-pr-cont",
					//	onclick : "selectItem()",

				}).appendTo(item_div);

			var h4 = jQuery('<h4/>', {
					class : "list-group-item-heading",
					text : itemData['popupHeading'],
				}).appendTo(div_pr_cont);

			var p = jQuery('<p/>', {
					class : "list-group-item-text ",
					html : itemData['popupText']
				}).appendTo(div_pr_cont)

				var nextEvent = navigatorCore.navItemFLayer.getNextEvent(itemData['id']);
			if (nextEvent) {
				var info = jQuery('<span/>', {
						class : "item-list-event-info",
					}).appendTo(div_pr_cont);

				var span1 = jQuery('<span/>', {
						class : "item-list-event-type",
						text : nextEvent.type + ':'
					}).appendTo(info);

				var timeTo = displayTimeFromSecondsToHoursAndMinutes(nextEvent.start_time_sec - getSecondsFromDayStart(new Date()))
					//console.log('timeTo ' + timeTo);
					var span2 = jQuery('<span/>', {
						class : "item-list-event-timeto",
						text : "in " + timeTo
					}).appendTo(info);

				var span3 = jQuery('<span/>', {
						class : "item-list-event-time",
						text : nextEvent.start_time
					}).appendTo(info);

				var span4 = jQuery('<span/>', {
						class : "item-list-event-status",
						text : nextEvent.status
					}).appendTo(info);

				if (nextEvent.status == 'open')
					span4.addClass('event-open')

					if (nextEvent.status == 'closed')
						span4.addClass('event-closed')

			}

			var a_dir = jQuery('<a/>', {
					href : '#',
					class : "item-list-get-directions-link",
					onclick : "getDirections(event)",
					text : langString (language, 'directions')
				}).appendTo(div_pr_cont);

			a_dir.attr('target-id', itemData['id']);

		}

	})
	
	




	$('#trailEvents').on('show.bs.modal', function (event) {
		//var a = $(event.relatedTarget) // Button that triggered the modal
		//console.log("you clicked " + );
		//	var targer_id = $(event.relatedTarget).attr('target_id');
		//	var itemData = navigatorCore.navItemFLayer.boundItemsData[targer_id];

		var modal = $(this)
			var div = modal.find('#events-list-display');
		$(div).empty();
		var d = new Date();
		var day = d.getDay();

		var eventsData = navigatorCore.navItemFLayer.eventsDataByDay[day];
		//console.log("eventData.eventsData" + JSON.stringify(eventsData))
		for (var i = 0; i < eventsData.length; i++) {
			var eventData = eventsData[i];
			//console.log("eventData.location_id" +eventData.location_id  + " " +eventData.start_time + '>' +eventData.start_time_sec)
			var itemData = navigatorCore.navItemFLayer.boundItemsData[eventData.location_id];
			
			if (itemData) {
				var item_div = jQuery('<div/>', {
						//	href : '#',
						class : "list-group-item",
						//	onclick : "selectItem()",

					}).appendTo(div);

				item_div.attr('target-id', itemData.id);

				var div_img_cont = jQuery('<div/>', {
						//	href : '#',
						class : "item-list-img-cont",
						//	onclick : "selectItem()",

					}).appendTo(item_div);

				var img = jQuery('<img/>', {
						class : "pull-left item-list-img",
						src : itemData['popupPreviewURL']
					}).appendTo(div_img_cont);

				var distanceTo = navigatorCore.findDistanceToItemM(itemData['id']);

				var span = jQuery('<span/>', {
						class : "label label-success label-pill",
						text : distanceTo
					}).appendTo(div_img_cont)

					var div_pr_cont = jQuery('<div/>', {
						//	href : '#',
						class : "item-list-pr-cont",
						//	onclick : "selectItem()",

					}).appendTo(item_div);

				var h4 = jQuery('<h4/>', {
						class : "list-group-item-heading",
						text : eventData['name'],
					}).appendTo(div_pr_cont);

				var nextEvent = eventData;
				if (nextEvent) {
					var info = jQuery('<span/>', {
							class : "item-list-event-info",
						}).appendTo(div_pr_cont);

					var span1 = jQuery('<span/>', {
							class : "item-list-event-type",
							text : nextEvent.type + ':'
						}).appendTo(info);

					var timeTo = displayTimeFromSecondsToHoursAndMinutes(nextEvent.start_time_sec - getSecondsFromDayStart(new Date()))
						//console.log('timeTo ' + timeTo);
						if (timeTo)
							var span2 = jQuery('<span/>', {
									class : "item-list-event-timeto",
									text : "in " + timeTo
								}).appendTo(info);

						var span3 = jQuery('<span/>', {
							class : "item-list-event-time",
							text : nextEvent.start_time
						}).appendTo(info);

					var span4 = jQuery('<span/>', {
							class : "item-list-event-status",
							text : nextEvent.status
						}).appendTo(info);

					if (nextEvent.status == 'open')
						span4.addClass('event-open')

						if (nextEvent.status == 'closed')
							span4.addClass('event-closed')

				}

				var a_dir = jQuery('<a/>', {
						href : '#',
						class : "item-list-get-directions-link",
						onclick : "getDirections(event)",
						text : langString (language, 'directions')
					}).appendTo(div_pr_cont);

				a_dir.attr('target-id', itemData['id']);

			}
		}

	})
}

function getDirections(e) {
	//alert('getDirections'+e );
	
	var target_id = e.target.getAttribute('target-id');
	if (target_id) {
		//console.log('You selected: ' + target_id);
		getDirectionsByID(target_id);
	}
}

function getDirectionsByID(target_id) {
   // resetMap();
    navigatorCore.getDirectionsForTargetItem(target_id);
 //   if (navigatorCore.navPathAssistant.isActive) {
      //  if (navigatorCore.locationManager.getLastKnownLocation())
          //  map.getView().setCenter(navigatorCore.locationManager.getLastKnownLocation());
   // }
}


function createMapPoupHTML(itemID) {

	var itemData = navigatorCore.navItemFLayer.boundItemsData[itemID];

	var div = jQuery('<div/>', {})



	if (itemData['location_id']) {
		a = jQuery('<a/>', {
				//id : 'foo',
				href : "#",
				onclick : "appConnector.locationViewMoreClicked(event);",
				location_id : itemData['location_id'],
				id : "item-map-popup-link",
				class : "item-map-popup-link"
			}).appendTo(div);
			
	} else  {
				a = jQuery('<div/>', {
			//id : 'foo',
			//href : '#',
			id : "item-map-popup-link",
			class : "item-map-popup-link"
		}).appendTo(div);
		
	}	
		
		
	//a.attr('target-id', itemID);

	var divImg = jQuery('<div/>', {
			href : '#',
			id : "item-map-popup-img",
			class : "item-map-popup-img"
		}).appendTo(a);

	var img = jQuery('<img/>', {
			id : "item-map-popup-img-img",
			class : "item-map-popup-img-img",
			src : itemData['popupPreviewURL']
		}).appendTo(divImg);
		
	if (itemData['location_id']) {
		$(img).attr('location_id', itemData['location_id']);
	}	
		
	var divTitleBar = jQuery('<div/>', {
			href : '#',
			id : "item-map-popup-title-bar",
			class : "item-map-popup-title-bar"
	}).appendTo(div);


	var span = jQuery('<span/>', {
			id : "item-map-popup-text",
			class : "item-map-popup-text",
			text : itemData['popupHeading']
		}).appendTo(divTitleBar);





		var nextEvent = navigatorCore.navItemFLayer.getNextEvent(itemID);
	   if (nextEvent) {



		var span1 = jQuery('<span/>', {
				id : "item-map-popup-event-type",
				class : "item-map-popup-event-type",
				text : nextEvent.type + ' at: '
			}).appendTo(divTitleBar);

		var span3 = jQuery('<span/>', {
				id : "item-map-popup-event-time",
				class : "item-map-popup-event-time",
				text : nextEvent.start_time
			}).appendTo(divTitleBar);


	}
	jQuery('<br/>', {}).appendTo(span);
	
	var distanceTo = navigatorCore.findDistanceToItemM(itemData['id']);
	if (distanceTo)
		var spanDistance = jQuery('<span/>', {
				id : "item-map-popup-distance",
				class : "item-map-popup-distance",
				text : distanceTo
			}).appendTo(divImg);
			
	var visited = tourItems && tourItems[itemData['id']] && tourItems[itemData['id']]['s']==1;		
	if(visited){

			var span = jQuery('<span/>', {
					class : "label label-info label-pill",
					text : 'visited'
				}).appendTo(divImg)
		}		
	var span_bottom_toolbar = jQuery('<span/>', {
			id : "item-map-popup-bottom-toolbar",
			class : "item-map-popup-bottom-toolbar"
	}).appendTo(div);
	if(navigatorCore.showGetDirectionsLink(itemData['coord'])){
		var a_dir = jQuery('<a/>', {
				href : '#',
				id : "item-map-popup-get-directions-link",
				class : "item-map-popup-get-directions-link",
				onclick : "getDirections(event)",
				text : langString(language,'directions')
			}).appendTo(span_bottom_toolbar);

		a_dir.attr('target-id', itemID);
	}
	if(itemData['location_id'] ){

		var link = jQuery('<a/>', {
			href : "#",
			onclick : "appConnector.locationViewMoreClicked(event);",
			location_id : itemData['location_id'],
			class : "item-map-popup-view-more",
			id : "item-map-popup-view-more",
			//class : "item-map-popup-get-directions-link",
			text :  langString (language, 'info')
		}).appendTo(span_bottom_toolbar);
	}
	
	if(canBeAddedToMyDay(itemData.marker_type)){

	var link = jQuery('<a/>', {
				href : "#",
				onclick : "appConnector.locationAddToMyDayClicked(event);",
				location_id : itemData['location_id'],
				markerid : itemData['id'],
				class : "item-map-popup-add-to-my-day-link",
				id : "item-map-popup-add-to-my-day-link",
				text :  langString(language,'add')
	}).appendTo(span_bottom_toolbar);
	
	} 
	return div.html();
}

function dismissEye() {
	navigatorCore.lookAssistant.setActive(false);
	navigatorCore.lookAssistant.cleanup();
	$("#eye_overlay").hide();
	if (navigatorCore.navPathAssistant.isActive) {
		$("#navigation_overlay").show();
	}
}
function selectItem() {
	$('#trailItems').modal('toggle');
	$("#main_bar").hide();
	$("#navigation_overlay").show();
	$("#selection_overlay").show();
	$("#selection_overlay").show();

}

function selectItemSearch() {
	$('#trailItemsSearch').modal('toggle');
	$("#main_bar").hide();
	$("#navigation_overlay").show();
	$("#selection_overlay").show();

}
//generateTestEventData();

function generateTestEventData() {
	var eventsList = {};
	var eventDays = [0, 1, 2, 3, 4, 5, 6];
	var eventData = {
		id : '1',
		type : 'Sale',
		name : 'Sale 30% off',
		location_id : '1',
		days : eventDays,
		start_time : '14:30',
		status : 'not started'
	}

	eventsList[eventData.id] = eventData;

	eventDays = [0, 1, 2, 3, 4, 5, 6];
	eventData = {
		id : '2',
		type : 'Promotion',
		name : 'Wine tasting',
		location_id : '2',
		days : eventDays,
		start_time : '16:30',
		status : 'not started'
	}

	eventsList[eventData.id] = eventData;

	//console.log(JSON.stringify(eventsList));

}

function generateTestKiosData() {
	var productData = {};
	var productDataForCategory = [];
	productData['category-1'] = productDataForCategory
	var eventData = {
		id : 'test-01',
		type : 'Product',
		name : 'Tomato Saurce',
		category_id : 'category-1',
		index : '1',
		m_name : 'Master Foods',
		m_email : 'mf@somehost.com',
		status : 'green'
	}

	//productDataForCategory[eventData.id] = eventData;
 productDataForCategory.push(eventData);
 
 
 eventData = {
		id : 'test-02',
		type : 'Product',
		name : 'Pasta',
		category_id : 'category-1',
		index : '2',
		m_name : 'Master Foods',
		m_email : 'mf@somehost.com',
		status : 'gold'
	}

	//productDataForCategory[eventData.id] = eventData;
 productDataForCategory.push(eventData);
	
	 eventData = {
		id : 'test-03',
		type : 'Product',
		name : 'Instant Noodles',
		category_id : 'category-1',
		index : '3',
		m_name : 'Master Foods',
		m_email : 'mf@somehost.com',
		status : 'red'
	}

	//productDataForCategory[eventData.id] = eventData;
 productDataForCategory.push(eventData);
	eventData = {
		id : 'test-04',
		type : 'Product',
		name : 'Savoy Original',
		category_id : 'category-1',
		index : '4',
		m_name : 'X Foods',
		m_email : 'x@somehost.com',
		status : 'gold'
	}

	//productDataForCategory[eventData.id] = eventData;
 productDataForCategory.push(eventData);
	//console.log(JSON.stringify(productData));

}

function generateTestKiosDataC() {
	var productData = [];

	var eventData = {
		id : 'category-1',
		type : 'Category',
		name : 'Category 1',
		index : '1',
	}

	//productDataForCategory[eventData.id] = eventData;
 productData.push(eventData);

 eventData = {
		id : 'category-2',
		type : 'Category',
		name : 'Category 2',
		index : '2',
	}
	//productDataForCategory[eventData.id] = eventData;
 productData.push(eventData);
	
	 eventData = {
		id : 'category-3',
		type : 'Category',
		name : 'Category 3',
		index : '3',
	}

	//productDataForCategory[eventData.id] = eventData;
 productData.push(eventData);
	eventData = {
		id : 'category-4',
		type : 'Category',
		name : 'Category 4',
		index : '4',
	}

	//productDataForCategory[eventData.id] = eventData;
 productData.push(eventData);
	//console.log(JSON.stringify(productData));

}


function generateTestKiosDataT() {
	var productData = {};

	var eventData = {
		id : 'gold',
		name : 'This is gold template title',
		text : 'This is gold template text...',
	}

	productData[eventData.id] = eventData;

	var eventData = {
		id : 'green',
		name : 'This is green template title',
		text : 'This is green template text...',
	}
	productData[eventData.id] = eventData;
	
	var eventData = {
		id : 'red',
		name : 'This is red template title',
		text : 'This is red template text...',
	}


	productData[eventData.id] = eventData;
	
	
//	console.log(JSON.stringify(productData));

}
//simulate beacon signals
//[16833198.323499024,-4010794.9400065867]
//[16833184.364783894,-4010778.6672798623]
//[16833178.39314106,-4010804.9425083348]
//[16833178.39314106,-4010804.9425083348]
/*var testData = [{
		'id' : 'SM90859',
		'rm' : 23
	}, {
		'id' : 'SM84591',
		'rm' : 15
	}, {
		'id' : 'SM50580',
		'rm' :17
	}, {
		'id' : 'SM74590',
		'rm' : 22
	}
];
*/
var testData = [{
		'location_id' : 226,
		//'id' : 'SM90859',
		'rm' : 4
	}
];
//console.log(testData)
/*
 var userLocationTest = setInterval(function () {
		simulateUserLocation()
	}, 3000);


*/


function simulateUserLocation() {
	console.log('simulate')
		//testData[0]['rm'] = testData[0]['rm']  + 1;
		//testData[1]['rm'] = testData[1]['rm']  + 1;
		//testData[2]['rm'] = testData[2]['rm']  - 1;
		//testData[3]['rm'] = testData[3]['rm']  -1;
		//testData[4]['rm']=Math.floor((Math.random() * 15) + 1);
		//[151.21651902794835,-33.86581659238466]
		//138.991090503609,-35.0824562621598
		//139.45170944470593,-34.51096342722799
		updateUserLocation(-34.51096342722799,139.45170944470593,testData);
	
}

function stopUserLocationTest() {
	clearInterval(userLocationTest);
}




function initDebugConsole(enable) {

	$("#js_console").hide();
if(enable){
	$("#js_console").show();
	console.log = debugLog;
	window.console.errors = debugLog;
	window.console.info = debugLog;
	window.console.warn = debugLog;
	window.onerror = debugLog;
	}


}

function updateUserLocation(lat,lng, beaconData) {
	//console.log("eventData.start_time " + eventData.start_time + '>' + eventData.start_time_sec);
	//console.log("lat = " + lat + "lng = " + lng);
	/*var log = document.getElementById('js_console');

	 var p = document.createElement('p');
	p.textContent = "lat = "+lat +"lgn=" +  lng;
	log.prependChild(p);*/
	//if(mapViewerOptions.positioningMode == 'external'){

		//appConnector.updateUserLocation([lat,lng]);

		appConnector.updateUserLocationLonLat([lng,lat],beaconData);

	//}
}

function debugLog(message, source, lineno, colno, error) {
	var log = document.getElementById('js_console');

	var p = document.createElement('p');
	p.textContent = ''
		if (message)
			p.textContent += message;
		if (source)
			p.textContent += " source: " + source;
		if (lineno)
			p.textContent += " line: " + lineno;
		if (colno)
			p.textContent += " col: " + colno;
		if (error)
			p.textContent += " error:" + error;
		if (error)
			p.textContent += " error:" + error;
		//	if(error && error.stack) p.textContent+=" error stack:"+ error.stack;

		log.prependChild(p);
}

Element.prototype.prependChild = function (child) {
	this.insertBefore(child, this.firstChild);
};

function searchByTerm() {
	var searchTerm = $("#searchTerm").val();
    var searchTour = $("#searchButton").attr('isTour')==1;
 //console.log(searchTour);
//	if(searchTerm){
	//	var searchResults = navigatorCore.navItemFLayer.searchByTerm(searchTerm);
		//console.log(JSON.stringify(searchResults));
	//	displaySearchResults(searchResults);
	//}
	if(searchTour) {
	var searchResults = navigatorCore.navItemFLayer.searchTour(sortedTourItems,true,searchTerm);
	displaySearchResultsTour(searchResults.items);
		
	} else {
		var searchResults = navigatorCore.navItemFLayer.searchByType(this.searchByTypeCache,true,searchTerm);
	displaySearchResults(searchResults);	
		
	}

	
}

function searchByType(type,strict) {
	// console.log('searchByType');
	$('#search_options').hide()
	$('#keyword_search').show()
	$("#searchTerm").val('');
	$("#searchButton").attr('isTour', 0);
	this.searchByTypeCache=type;
	
	var searchResults = navigatorCore.navItemFLayer.searchByType(type,strict);
	//console.log(JSON.stringify(searchResults));
	displaySearchResults(searchResults);
}



this.searchByTypeCache;


function searchByTypeEvents(type) {
	$('#search_options').show()
	$('#keyword_search').hide()
}

function resetSearch() {
		$('#search_options').show()
	    $('#keyword_search').hide()
		var modal = $('#trailItemsSearch')
		var div = modal.find('#items-list-display');
		$(div).empty();
}





function displaySearchResults(searchResults)
{
	//s console.log("final_items " + searchResults);
	var modal = $('#trailItemsSearch')
	var searchTerm = modal.find('#searchTerm');
	$(searchTerm).attr('placeholder',langString (language, 'search_for'));
	var searchButton = modal.find('#searchButton');
	$(searchButton).text(langString (language, 'search_button'));
	
	var cont_div = modal.find('#items-list-display');
	$(cont_div).empty();
	
	
	var count_div = jQuery('<div/>', {
		text: langString (language, 'search_found') + " " + searchResults.items.length + " " + langString (language, 'search_items'),
		class: "search-results-title"
	}).appendTo(cont_div);
	
	var div = jQuery('<div/>', {
		class: "results-list",
		id: "results-list"
	}).appendTo(cont_div);
	

//	var br_div = jQuery('<br/>', {}).appendTo(div);

	for (var key in searchResults.items) {
		var itemData = searchResults.items[key];

		var item_div = jQuery('<div/>', {
				//	href : '#',
				class : "search-results-item",
				//	onclick : "selectItem()",

			}).appendTo(div);

		item_div.attr('target-id', itemData.id);

		var div_img_cont = jQuery('<div/>', {
					//	href : '#',
					class : "item-list-img-cont",
					//	onclick : "selectItem()",

				}).appendTo(item_div);
				
       var image_cont =div_img_cont;

		   var showOnMapFunctionName = "showStaticMarkerOnMap('";
			if (mapViewerOptions.kioskMode) showOnMapFunctionName = "showStaticMarkerOnMapKiosk('";
			image_cont = jQuery('<a/>', {
					//id : 'foo',
					href: "#",
					// onclick : "appConnector.locationViewMoreClicked(event);",
					onclick : showOnMapFunctionName + itemData['id']+"')",
					location_id : itemData['location_id'],
					id: "item-list-img-cont",
					class: "item-list-img-cont"
				}).appendTo(div_img_cont);
				
		
				
			var img = jQuery('<img/>', {
					class : "pull-left item-list-img",
					src : itemData['popupPreviewURL']
				}).appendTo(image_cont);
			if (itemData['location_id']) {
				$(img).attr('location_id', itemData['location_id']);
			}	
			var distanceTo = navigatorCore.findDistanceToItemM(itemData['id']);
				//	var distanceTo;
			if(distanceTo){
			var span = jQuery('<span/>', {
					class : "label label-success label-pill",
					text : distanceTo
				}).appendTo(image_cont)
				}
				var div_pr_cont = jQuery('<div/>', {
					//	href : '#',
					class : "item-list-pr-cont",
					//	onclick : "selectItem()",

				}).appendTo(item_div);

				
					
				var divTitleBar = jQuery('<div/>', {
						href : '#',
						id : "item-map-search-title-bar",
						class : "item-map-search-title-bar"
				}).appendTo(item_div);


				var span = jQuery('<span/>', {
						id : "item-map-popup-text",
						class : "item-map-search-text",
						text : itemData['popupHeading']
					}).appendTo(divTitleBar);


	
				var nextEvent = navigatorCore.navItemFLayer.getNextEvent(itemData['id']);
			if (nextEvent) {
				var info = jQuery('<span/>', {
						class : "item-list-event-info",
					}).appendTo(div_pr_cont);

				var span1 = jQuery('<span/>', {
						class : "item-list-event-type",
						text : nextEvent.type + ':'
					}).appendTo(info);

				var timeTo = displayTimeFromSecondsToHoursAndMinutes(nextEvent.start_time_sec - getSecondsFromDayStart(new Date()))
					//console.log('timeTo ' + timeTo);
					var span2 = jQuery('<span/>', {
						class : "item-list-event-timeto",
						text : "in " + timeTo
					}).appendTo(info);

				var span3 = jQuery('<span/>', {
						class : "item-list-event-time",
						text : nextEvent.start_time
					}).appendTo(info);

				var span4 = jQuery('<span/>', {
						class : "item-list-event-status",
						text : nextEvent.status
					}).appendTo(info);

				if (nextEvent.status == 'open')
					span4.addClass('event-open')

					if (nextEvent.status == 'closed')
						span4.addClass('event-closed')

			}


			
		var span_bottom_toolbar = jQuery('<span/>', {
				id : "item-map-search-bottom-toolbar",
				class : "item-map-search-bottom-toolbar"
		}).appendTo(item_div);
		
	
		//var a_dir = jQuery('<a/>', {
				//	href : '#',
			//		class : "item-list-get-directions-link",
			//		onclick : "getDirections(event)",
			//		text : langString (language, 'directions')
			//	}).appendTo(span_bottom_toolbar);

		//	a_dir.attr('target-id', itemData['id']);
		
			var link = jQuery('<a/>', 
			{
						href : "#",
						class : "item-map-search-show-on-map",
						onclick : showOnMapFunctionName+ itemData['id']+"')",
						text : langString (language, 'show_on_map')
			}).appendTo(span_bottom_toolbar);		
			
		if (itemData['location_id']) {
			var link = jQuery('<a/>', {
					href: "#",
					onclick : "appConnector.locationViewMoreClicked(event);",
					location_id : itemData['location_id'],
					class: "item-map-search-view-more",
					id: "item-map-search-view-more",
					//class : "item-map-popup-get-directions-link",
					text: langString(language, 'info')
				}).appendTo(span_bottom_toolbar);

		}
		if (canBeAddedToMyDay(itemData.marker_type)) {

			var link = jQuery('<a/>', {
					href: "#",
					onclick : "appConnector.locationAddToMyDayClicked(event);",
					location_id : itemData['location_id'],
					markerid : itemData['id'],
					class: "item-map-search-add-to-my-day-link",
					id: "item-map-search-add-to-my-day-link",
					text: 'Add'
				}).appendTo(span_bottom_toolbar);

		}
		}

	
}



function findTypeOnMap(target_id,disableOtherTypes) {
	resetMap();
	resetSearch();
	if (disableOtherTypes) {
		for (var key in searchOptionsDef) {
			var searchOption = searchOptionsDef[key];
			var toggleID = '#' + searchOption.id + '-toggle';
			//console.log(toggleID)
			if (searchOption.id != target_id) {
				//toggleMarkers(searchOption.id, false);
				//console.log(toggleID)
				if ($(toggleID).attr('checked') == 'checked')
					$(toggleID).click()
			} else {
				//	toggleMarkers(searchOption.id, true);
				//	console.log(toggleID)
				if ($(toggleID).attr('checked') != 'checked')
					$(toggleID).click()
			}

		}
	}
	if ($('#trailItemsSearch').css('display') == 'none') {
		$("#trailItemsSearch .close").click();
		// 
		var modal = $('#trailItemsSearch')
			var div = modal.find('#items-list-display');
		$(div).empty();
		//$('#trailItemsSearch').modal('toggle');
		$("#btnSearch").click();
		var title = $('#trailItemsSearch').find('#search-modal-title')
		title.text(langString(language, 'search_modal_name'));
		$('#search_options').hide()
		$('#keyword_search').show()
		$("#searchTerm").val('');
		$("#searchButton").attr('isTour', 0);
		//this.searchByTypeCache = searchOptionsDef[target_id].types;
		//console.log(JSON.stringify(searchOptionsDef[target_id].types));
		searchByType(searchOptionsDef[target_id].types, true);

	}

	
	//$("#trailItemsSearch .close").click();

	//$("#btnSearch").click();


}

   
function findOnMap(){
	resetMap();
	resetSearch();
    if ($('#trailItemsSearch').css('display') == 'none') {
		$("#trailItemsSearch .close").click();
        $("#btnSearch").click();
		//$('#keyword_search').hide()
    }else{

	}

    //$("#trailItemsSearch .close").click();
	
	//$("#btnSearch").click();
	

}

function resetMap() {
	$(document.body).removeClass('tour');
	resetMapTour();
	removeTourUI();
	resetLabels();
	hideOnMapTempMarker();
	resetFilterOptionsToDefault();
	navigatorCore.tourPathAssistant.clearTourPathsCache();
	navigatorCore.navPathAssistant.stopNavigationAssistance()
	$("#navigation_overlay").hide();
	dismissEye();
	popup.hide();
	$("#item-details-modal-sm .close").click();
	$("#trailItemsSearch .close").click();
	$("#tourItems .close").click();
	$("#warning-modal-sm .close").click();

	if (mapViewerOptions.showEye) {
		$('#btnEye1').show()
	}
	if (mapViewerOptions.showSearchButton) {
		$('#btnSearch').show()
	}
	setMode(mapViewerOptions.mapMode);
}


function showOnMap(location_id){
	resetMap();
	var itemData = navigatorCore.navItemFLayer.boundItemsData_Location[location_id];
	var feature = itemData['feature']
	if (feature && feature.get("type") == "marker-with-popup") {
			insureTypeVisible(itemData.marker_type);
			//console.log(feature.get("type"));
			var coord = feature.getGeometry().getCoordinates();
			/* var props = feature.getProperties(); */
			//console.log('You popup_content: ' + feature.get('popup_content'));
			popup.show(coord, createMapPoupHTML(feature.getId()));
			map.getView().setCenter(feature.getGeometry().getCoordinates());
			map.getView().setZoom(mapViewerOptions.navZoom);

		}

}

function insureTypeVisible(marker_type) {
	var togglesArray = searchOptionsTypeLookupDic[marker_type]
		if (togglesArray) {
			//console.log(JSON.stringify(togglesArray));
			for (var keyT in togglesArray) {
				var toggleID = '#' + togglesArray[keyT];
				if ($(toggleID).attr('checked') != 'checked')
					$(toggleID).click()
			}
		}

}


function showStaticMarkerOnMap(marker_id){
	resetMap();
	var itemData = navigatorCore.navItemFLayer.boundItemsData[marker_id];
	var feature = itemData['feature']
	if (feature ) {
		insureTypeVisible(itemData.marker_type);
		
		var coord = feature.getGeometry().getCoordinates();
		var props = feature.getProperties();

		popup.show(coord, createMapPoupHTML(feature.getId()));
					//map.getView().setZoom(mapViewerOptions.navZoom);//
			//map.getView().setZoom(mapViewerOptions.navZoom);//
		map.getView().setCenter(feature.getGeometry().getCoordinates());
					setMode(mode_unlockAndRotateIcon);

	}

}


function showStaticMarkerOnMapKiosk(marker_id){
	//resetMap();
	var itemData = navigatorCore.navItemFLayer.boundItemsData[marker_id];
	var feature = itemData['feature']
	if (feature ) {
		insureTypeVisible(itemData.marker_type);
		
		var coord = feature.getGeometry().getCoordinates();
		var props = feature.getProperties();

		popup.show(coord, createMapPoupHTML(feature.getId()));
					map.getView().setZoom(mapViewerOptions.navZoom);
			map.getView().setZoom(mapViewerOptions.navZoom);
		map.getView().setCenter(feature.getGeometry().getCoordinates());
					setMode(mode_unlockAndRotateIcon);

	}

}

var searchOptionsDefDic = {};
var searchOptionsTypeLookupDic = {};

function initFilterOptions() {
	
	$("#searchTerm").on('input', function(){
      searchByTerm();
});


	var populated_types = {}

	var boundItemsData = navigatorCore.navItemFLayer.boundItemsData;
	for (var key in boundItemsData) {
		var itemData = boundItemsData[key];
		if (itemData.feature) {
			populated_types[itemData.marker_type] = "1"
		}
	}
	// create structure
	var search_options = $("#search_options")
	search_options.empty();
	// add tour option
	
	//adddTourItemsToFilter();
	

		for (var key in searchOptionsDef) {
			var searchOption = searchOptionsDef[key];
			searchOptionsDefDic[searchOption.id] = searchOption;
			
			var searchOptionRow = jQuery('<div/>', {
					//id : 'foo',
					//href : '#',
					id : searchOption.id + "-row",
					class : 'row'
				}).appendTo(search_options);

			var col1 = jQuery('<div/>', {
					class : 'col-xs-9'
				}).appendTo(searchOptionRow);

			var buttonSearch = jQuery('<button/>', {
					id : searchOption.id + "-toggle-button",
					class : 'btn btn-default btn-search',
					target_id : key,
				}).appendTo(col1);

			var img = jQuery('<img/>', {
					src : searchOption.icon,
					class : 'img-search-option',
				}).appendTo(buttonSearch);

			buttonSearch.append(searchOption.name);

			buttonSearch.bind("click", function (event, ui) {
				var targer_id = $(event.target).attr('target_id');
				//	console.log(event.target);
				//	console.log(targer_id);
				searchByType(searchOptionsDef[targer_id].types, true);

			});

			var col2 = jQuery('<div/>', {
					class : 'col-xs-3'
				}).appendTo(searchOptionRow);

			toggle = jQuery('<input/>', {
					id : searchOption.id + "-toggle",
					type : 'checkbox',
					class : "toggle",
					checked : "checked",
					target_id : key,
				}).appendTo(col2);
			toggle.bind("change", function (event, ui) {
				var targer_id = $(event.target).attr('target_id');
				//console.log(event.target);
				//console.log(targer_id);
				toggleMarkers(searchOptionsDef[targer_id].id, this.checked);
			});

			if (searchOption.disabled == true) {
				toggleMarkers(searchOption.id, false);
			}

		}

		for (var key in searchOptionsDef) {
			var searchOption = searchOptionsDef[key];
			var toggleEl = document.getElementById(searchOption.id + "-toggle")
			new Switchery(toggleEl);
			var hasItems = false;
			for (var keyT in searchOption.types) {
				var type = searchOption.types[keyT]
					//console.log('type: ' + type)
					//console.log('hasItems: ' + hasItems)
					if(!hasItems) hasItems = populated_types[type];
					var togglesArray = searchOptionsTypeLookupDic[type]
					if(!togglesArray) {
						togglesArray = [];
						searchOptionsTypeLookupDic[type]=togglesArray
					}
					togglesArray.push(searchOption.id + "-toggle");
			}

			if (!hasItems || searchOption.hide) {
				//console.log('disable: '+key)
				var containerID = '#' + searchOption.id + '-row';
				$(containerID).hide();
			}
		}

		//appConnector.showOnMap('199');
}

var disabledTypes = {}
function toggleMarkers(id, show) {
	//console.log("name " + name);
	//console.log("show " + show);
	dismissEye();
	var containerID = '#' + id + '-toggle-row';
	var buttonID = '#' + id + '-toggle-button';
	var toggleID = '#' + id + '-toggle';
	
	var searchOption = searchOptionsDefDic[id]
		for (var keyT in searchOption.types) {
			var type = searchOption.types[keyT]
				//console.log('type: ' + type)
			if(show)	
			disabledTypes[type]=undefined;	
			else 
			disabledTypes[type]=true;

		}
	//console.log("$(toggleID).attr('checked') " + $(toggleID).attr('checked'));
	if (show && $(toggleID).attr('checked') == 'checked')
		return;

	if (show) {

		$(containerID).removeClass('disabled');
		$(buttonID).prop("disabled", false);
		$(toggleID).attr('checked', 'checked');
	} else {
		$(containerID).addClass('disabled');
		$(buttonID).prop("disabled", true);
		$(toggleID).attr('checked', null);
	}

	var searchOption = searchOptionsDefDic[id]
		for (var keyT in searchOption.types) {
			var type = searchOption.types[keyT]
				//console.log('type: ' + type)
				var boundItemsData = navigatorCore.navItemFLayer.boundItemsData;
			for (var key in boundItemsData) {
				var itemData = boundItemsData[key];
				if (itemData.feature && itemData.marker_type == type) {
					var marker_type = itemData.feature.get('marker_type')
					var style = markerStyles[marker_type]['normal'];			
					if (show)
						style.getImage().setOpacity('1');
					else
						style.getImage().setOpacity('0');
					itemData.feature.changed();
				}
			}
		}
		if (searchOption.linked_labels) {
			navigatorCore.vectorLabels.getSource().forEachFeature(
				function (feature) {
				var type = feature.get('type');
				// var isNav=feature.getId()&& feature.getId().indexOf('temp')==-1;
				// console.log("feature= " + type +' '+ feature.getId());
				if (type == "img-label") {
					if (!show)
						feature.set('hide','true')
					else
						feature.set('hide',undefined)
						//feature.getStyle().getImage().setOpacity('0');
				  	feature.changed();

				}

			})

		}
}




function getDirectionsWildSqaud(target_id) {

	resetMap();
	for (var key in searchOptionsDef) {
			var searchOption = searchOptionsDef[key];
				var toggleID = '#' + searchOption.id + '-toggle';
			if (searchOption.id != '12') {
				if ($(toggleID).attr('checked') == 'checked')
					$(toggleID).click()
				toggleMarkers(searchOption.id, false);

			} else {
				if ($(toggleID).attr('checked') != 'checked')
					$(toggleID).click()
				toggleMarkers(searchOption.id, true);

			}
   }			
	//alert('getDirections'+e );
	if (target_id) {
		//console.log('You selected: ' + target_id);
		navigatorCore.getDirectionsForTargetItem(target_id);
		if (navigatorCore.navPathAssistant.isActive) {
		$("#navigation_overlay").show();
			if (navigatorCore.locationManager.getLastKnownLocation())
				map.getView().setCenter(navigatorCore.locationManager.getLastKnownLocation());
		}
	}

}




var tempMarkerFeature;

function initShowOnMapTempMarker(vectorSourceMarkers){
 tempMarkerFeature = new ol.Feature({
			id : 'tempMarkerFeature',
			type : 'temp-marker-with-popup',
			geometry : new ol.geom.Point(mapCenter),
			marker_type : 'temp',
			name : 'temp marker'
		});
	tempMarkerFeature.setId('tempMarkerFeature');

	//tempMarkerFeature.setStyle(tempMarkerFeatureStyle);
	vectorSourceMarkers.addFeature(tempMarkerFeature);

}

var tempMarkerFeatureStyle;
	
function createTempMarkerMapPoupHTML(itemData) {

	var div = jQuery('<div/>', {class : "temp-item-map-popup-content"})

		a = jQuery('<a/>', {
			//id : 'foo',
			href : '#',
			onclick : "appConnector.tempMarkerViewMoreClicked(event);",
			extid : itemData['extid'],
			id : "temp-item-map-popup-link",
			class : "item-map-popup-link"
		}).appendTo(div);
		
		
	//a.attr('target-id', itemID);

	var divImg = jQuery('<div/>', {
			id : "temp-item-map-popup-img",
			class : "item-map-popup-img" + " " + itemData.itemType
		}).appendTo(a);

	var img = jQuery('<img/>', {
			id : "temp-item-map-popup-img-img",
			class : "item-map-popup-img-img"+ " " + itemData.itemType,
			src : itemData['popupPreviewURL'],
			extid : itemData['extid'],
		}).appendTo(divImg);
		
		
		
	var divTitleBar = jQuery('<div/>', {
			href : '#',
			id : "temp-item-map-popup-title-bar",
			class : "item-map-popup-title-bar"
	}).appendTo(div);


	var span = jQuery('<span/>', {
			id : "temp-item-map-popup-text",
			class : "item-map-popup-text",
			text : itemData['popupHeading']
		}).appendTo(divTitleBar);



	jQuery('<br/>', {}).appendTo(span);

			
			
	var span_bottom_toolbar = jQuery('<span/>', {
			id : "temp-item-map-popup-bottom-toolbar",
			class : "item-map-popup-bottom-toolbar"
	}).appendTo(div);

	var distanceTo =  navigatorCore.findDistanceToItemM(itemData['id']);
	if (distanceTo)
		var spanDistance = jQuery('<span/>', {
				id : "temp-item-map-popup-distance",
				class : "item-map-popup-distance",
				text : distanceTo
			}).appendTo(divImg);
			
	


	var a_dir = jQuery('<a/>', {
			href : '#',
			id : "temp-item-map-popup-get-directions-link",
			class : "item-map-popup-get-directions-link",
			onclick : "getDirections(event)",
			text : langString(language,'directions')
		}).appendTo(span_bottom_toolbar);

	a_dir.attr('target-id', itemData['id']);

	//if(itemData['location_id'] ){


	var link = jQuery('<a/>', {
			href: '#',
			class: "item-map-popup-view-more",
			id: "temp-item-map-popup-view-more",
			//class : "item-map-popup-get-directions-link",
			text: langString(language, 'info'),
			onclick : "appConnector.tempMarkerViewMoreClicked(event);",
			extid : itemData['extid'],
		}).appendTo(span_bottom_toolbar);

//	}

	return div.html();
}

function showOnMapTempMarker(lonLat, extid, itemType, popupHeading, popupText, popupPreviewURL){
	resetMap();
	
	var coord = ol.proj.fromLonLat(lonLat)
	
	if(!ol.extent.containsCoordinate(viewableBounds, coord)){
			appConnector.handleError('4',langString (language, 'warning_marker_outside_bounds'));
			return;
	}
		
	
	var style = markerStyles["temp"]['normal'];
	style.getImage().setOpacity('1');
	tempMarkerFeature.getGeometry().setCoordinates(coord);

	map.getView().setCenter(coord);
    var itemData = {
	'id' : 'temp',
	'marker_type' : 'temp',
	'extid' : extid,
	'itemType' : itemType,
	'popupHeading':popupHeading,
	'popupText':popupText,
	'popupPreviewURL' : popupPreviewURL,
	'coord': coord
	}

	navigatorCore.navItemFLayer.boundItemsData['temp']=itemData;
	tempMarkerFeature['itemData']=itemData;
	popup.show(coord, createTempMarkerMapPoupHTML(itemData));
}


function hideOnMapTempMarker(){	
//console.log('hideOnMapTempMarker');
var style = markerStyles["temp"]['normal'];
style.getImage().setOpacity('0');
tempMarkerFeature.changed();
	
}

function resetFilterOptions(flag){
	

			for (var key in searchOptionsDef) {
			var searchOption = searchOptionsDef[key];
			var toggleID = '#' + searchOption.id + '-toggle';
			if (!flag && $(toggleID).attr('checked') == 'checked')
			$(toggleID).click()	
			if (flag && $(toggleID).attr('checked')!= 'checked')
			$(toggleID).click()				
		}

}


function resetFilterOptionsToDefault(){
		for (var key in searchOptionsDef) {
			var searchOption = searchOptionsDef[key];
			var toggleID = '#' + searchOption.id + '-toggle';
			//console.log(toggleID)
			if (searchOption.disabled == true) {
				//toggleMarkers(searchOption.id, false);
				//console.log(toggleID)
				if ($(toggleID).attr('checked') == 'checked')
				$(toggleID).click()
			} else {
			//	toggleMarkers(searchOption.id, true);	
			//	console.log(toggleID)
				if ($(toggleID).attr('checked') != 'checked')
				$(toggleID).click()
			}	

		}

}

function setLanguage(lang) {
	language = lang;
	//resetMap();
	initSearchOptionsDef();
	initFilterOptions();

	var boundItemsData = navigatorCore.navItemFLayer.boundItemsData;
	for (var key in boundItemsData) {
		var itemData = boundItemsData[key];
		if (itemData['popupHeadingIntl'])
			itemData['popupHeading'] = itemData['popupHeadingIntl'][language]
				if (!itemData['popupHeading'])
					itemData['popupHeading'] = '';
				if (itemData['popupTextIntl'])
					itemData['popupText'] = itemData['popupTextIntl'][language]
	}

	var title = $('#calibrate-modal-sm').find('#calibrate-modal-title')
		title.text(langString(language, 'calibrate_modal_name'));

	var body = $('#calibrate-modal-sm').find('#calibrate-modal-body');
	body.html(langString(language, 'calibrate_modal_body'));

	var title = $('#trailItemsSearch').find('#search-modal-title')
		title.text(langString(language, 'search_modal_name'));

	var title = $("#eye_overlay").find('#eye_overlay_title')
		title.text(langString(language, 'eye_scanner_name'));

	$("#btnModeResume").text(langString(language, 'resume'));
}